@@ -6,11 +6,10 @@ marc2bfrdf -v -o /tmp/ph1.ttl -s /tmp/ph1.stats.js -b http://example.org test/re
 import re
 import os
 import json
-import time
 import logging
-import string
 import itertools
 import asyncio
+from collections import defaultdict
 
 #from datachef.ids import simple_hashstring
 from bibframe.contrib.datachefids import idgen, FROM_EMPTY_HASH
@@ -102,7 +101,7 @@ def isbn_instancegen(params):
         instance_ids.append(instanceid)
 
     for instanceid in instance_ids:
-        model.add(I(workid), I(iri.absolutize('hasInstance', vocabbase)), instanceid)
+        model.add(instanceid, I(iri.absolutize('instantiates', vocabbase)), I(workid))
         model.add(I(instanceid), TYPE_REL, I(iri.absolutize('Instance', vocabbase)))
 
     return instance_ids
@@ -120,18 +119,25 @@ def instance_postprocess(params):
 
 def marc_lookup(rec, fieldspecs):
     result = []
-    lookup_helper = dict(( f.split('$') for f in fieldspecs ))
+    lookup_helper = defaultdict(list)
+    for f in fieldspecs:
+        k, v = f.split('$')
+        lookup_helper[k].append(v)
+    #dict((  for f in fieldspecs ))
     #dict((target_code, target_sf = fieldspec.split('$')
     for row in rec:
         if row[0] == DATAFIELD:
             rowtype, code, xmlattrs, subfields = row
+            #print(code, ''.join(subfields.keys()), end='|')
             if code in lookup_helper:
-                result.append(subfields.get(lookup_helper[code], ''))
+                result.extend([ subfields.get(sf, []) for sf in lookup_helper[code] ])
+    result = list(itertools.chain.from_iterable(result))
     return result
 
 
 RECORD_HASH_KEY_FIELDS = [
-    '130$a', '240$a', '240$b', '240$c', '240$h', '245$a', '246$a', '246$i', '830$a', #Title info
+    '130$a', '240$a', '240$b', '240$c', '240$h', '245$a', '245$b', '245$n', '245$p', '246$a', '246$i', '246$n', '246$p', '830$a', #Title info
+    '250$a', '250$b', #Edition
     '100$a', '100$d', '100$q', '110$a', '110$c', '110$d', '111$a', '110$c', '110$d', #Creator info
     '700$a', '700$d', '710$a', '710$b', '710$d', #Contributor info
     '600$a', '610$a', '611$a', '650$a', '650$x', '651$a', '615$a', '690$a' #Subject info
@@ -143,7 +149,8 @@ def record_hash_key(rec):
 
 
 @asyncio.coroutine
-def record_handler(loop, relsink, entbase=None, vocabbase=BFZ, limiting=None, plugins=None, ids=None, postprocess=None, out=None, logger=logging, **kwargs):
+def record_handler(loop, relsink, entbase=None, vocabbase=BFZ, limiting=None, plugins=None,
+                   ids=None, postprocess=None, out=None, logger=logging, transforms=TRANSFORMS, **kwargs):
     '''
     loop - asyncio event loop
     entbase - base IRI used for IDs of generated entity resources
@@ -160,7 +167,7 @@ def record_handler(loop, relsink, entbase=None, vocabbase=BFZ, limiting=None, pl
     existing_ids = set()
     initialize(hashidgen=ids, existing_ids=existing_ids)
     #Start the process of writing out the JSON representation of the resulting Versa
-    out.write('[')
+    if out: out.write('[')
     first_record = True
     try:
         while True:
@@ -218,20 +225,28 @@ def record_handler(loop, relsink, entbase=None, vocabbase=BFZ, limiting=None, pl
                         lookup = '{0}-{1}{2}'.format(*((code,) + indicators))
                     for k, v in subfields.items():
                         lookup = '{0}${1}'.format(code, k)
-                        if lookup in TRANSFORMS: to_process.append((TRANSFORMS[lookup], v))
+                        for valitems in v:
+                            if lookup in transforms: to_process.append((transforms[lookup], valitems))
+
+                    if code in transforms:
+                        to_process.append((transforms[code], ''))
+                    else:
+                        params.setdefault('dropped_codes',{}).setdefault(code,0)
+                        params['dropped_codes'][code] += 1
 
-                    if code in TRANSFORMS: to_process.append((TRANSFORMS[code], ''))
                     #if code == '100':
                     #    logger.debug(to_process)
 
                     #Apply all the handlers that were found
-                    for func, val in to_process:
+                    for funcinfo, val in to_process:
+                        #Support multiple actions per lookup
+                        funcs = funcinfo if isinstance(funcinfo, tuple) else (funcinfo,)
                         #Build Versa processing context
-                        ctx = bfcontext(workid, code, [(workid, code, val, subfields)], relsink, base=vocabbase, hashidgen=ids, existing_ids=existing_ids)
-                        new_stmts = func(ctx, workid, instanceid)
-                        #FIXME: Use add
-                        for s in new_stmts: relsink.add(*s)
-                        #logger.debug('.')
+                        for func in funcs:
+                            ctx = bfcontext(workid, code, [(workid, code, val, subfields)], relsink, base=vocabbase, hashidgen=ids, existing_ids=existing_ids)
+                            new_stmts = func(ctx, workid, instanceid)
+                            #XXX: Use add_many?
+                            for s in new_stmts: relsink.add(*s)
 
                     if not to_process:
                         #Nothing else has handled this data field; go to the fallback
@@ -239,7 +254,8 @@ def record_handler(loop, relsink, entbase=None, vocabbase=BFZ, limiting=None, pl
                         for k, v in subfields.items():
                             fallback_rel = fallback_rel_base + k
                             #params['transforms'].append((code, fallback_rel))
-                            relsink.add(I(workid), I(iri.absolutize(fallback_rel, vocabbase)), v)
+                            for valitem in v:
+                                relsink.add(I(workid), I(iri.absolutize(fallback_rel, vocabbase)), valitem)
 
                 params['code'] = code
 
@@ -274,18 +290,20 @@ def record_handler(loop, relsink, entbase=None, vocabbase=BFZ, limiting=None, pl
                 #yield from asyncio.async(asyncio.sleep(0.01))
                 #yield from asyncio.sleep(0.01) #Basically yield to next task
 
-            if not first_record: out.write(',\n')
-            first_record = False
-            last_chunk = None
-            #Using iterencode avoids building a big JSON string in memory, or having to resort to file pointer seeking
-            #Then again builds a big list in memory, so still working on opt here
-            for chunk in json.JSONEncoder().iterencode([ link for link in relsink ]):
-                if last_chunk is None:
-                    last_chunk = chunk[1:]
-                else:
-                    out.write(last_chunk)
-                    last_chunk = chunk
-            if last_chunk: out.write(last_chunk[:-1])
+            #Can we somehow move this to passed-in postprocessing?
+            if out and not first_record: out.write(',\n')
+            if out:
+                first_record = False
+                last_chunk = None
+                #Using iterencode avoids building a big JSON string in memory, or having to resort to file pointer seeking
+                #Then again builds a big list in memory, so still working on opt here
+                for chunk in json.JSONEncoder().iterencode([ link for link in relsink ]):
+                    if last_chunk is None:
+                        last_chunk = chunk[1:]
+                    else:
+                        out.write(last_chunk)
+                        last_chunk = chunk
+                if last_chunk: out.write(last_chunk[:-1])
             #FIXME: Postprocessing should probably be a task too
             if postprocess: postprocess(rec)
             #limiting--running count of records processed versus the max number, if any
@@ -294,7 +312,7 @@ def record_handler(loop, relsink, entbase=None, vocabbase=BFZ, limiting=None, pl
                 break
     except GeneratorExit:
         logger.debug('Completed processing {0} record{1}.'.format(limiting[0], '' if limiting[0] == 1 else 's'))
-        out.write(']')
+        if out: out.write(']')
 
         #if not plugins: loop.stop()
         for plugin in plugins:
