@@ -18,7 +18,7 @@ from bibframe.reader.util import *
 # where do we put LDR info, e.g. LDR 07 / 19 positions = mode of issuance
 #Don't do a simple field renaming of ISBN because
 
-TRANSFORMS = {
+BFLITE_TRANSFORMS = {
     #Link to the 010a value, naming the relationship 'lccn'
     '010$a': onwork.rename(rel='lccn'),
     '017$a': onwork.rename(rel='legalDeposit'),
@@ -39,11 +39,11 @@ TRANSFORMS = {
     '050$a': onwork.rename(rel='lcCallNumber'),
     '050$b': onwork.rename(rel='lcItemNumber'),
     '050$3': onwork.rename(rel='material'),
-    '060$a': oninstance.rename(rel='nlmCallNumber'),
-    '060$b': oninstance.rename(rel='nlmItemNumber'),
+    '060$a': onwork.rename(rel='nlmCallNumber'),
+    '060$b': onwork.rename(rel='nlmItemNumber'),
     '061$a': onwork.rename(rel='nlmCopyStatement'),
-    '070$a': oninstance.rename(rel='nalCallNumber'),
-    '070$b': oninstance.rename(rel='nalItemNumber'),
+    '070$a': onwork.rename(rel='nalCallNumber'),
+    '070$b': onwork.rename(rel='nalItemNumber'),
     '071$a': onwork.rename(rel='nalCopyStatement'),
     '082$a': onwork.rename(rel='deweyNumber'),
 
@@ -58,7 +58,7 @@ TRANSFORMS = {
     '110': onwork.materialize('Organization', 
                               values('creator', normalizeparse(subfield('e')), normalizeparse(subfield('4'))), 
                               unique=values(subfield('a'), subfield('b'), subfield('c'), subfield('d'), subfield('g'), subfield('j'), subfield('q'), subfield('u')), 
-                              mr_properties={'name': subfield('a'), 'date': subfield('d'), 'hasAuthorityLink': subfield('0')}),
+                              mr_properties={'name': subfield('a'), 'subordinateUnit': subfield('b'), 'date': subfield('d'), 'hasAuthorityLink': subfield('0')}),
 
     '111': onwork.materialize('Meeting', 
                               values('creator', normalizeparse(subfield('e')), normalizeparse(subfield('4'))), 
@@ -68,19 +68,24 @@ TRANSFORMS = {
     '210$a': oninstance.rename(rel='abbreviatedTitle'),
     '222$a': oninstance.rename(rel='keyTitle'),
 
-    '240$a': onwork.materialize('Collection', 
-                                'uniformMemberOf', 
-                                unique=all_subfields, 
-                                mr_properties={'name': subfield('a'), 'legalDate': subfield('d'), 'medium': subfield('h'), 'musicMedium': subfield('m'), 'musicKey': subfield('r')}),
+    '240': onwork.materialize('Collection', 
+                              'memberOf', 
+                              unique=values(subfield('a'), subfield('h'), subfield('k'), subfield('l'), subfield('m'), subfield('s')), 
+                              mr_properties={'title': subfield('a'), 'legalDate': subfield('d'), 'medium': subfield('h'), 'musicMedium': subfield('m'), 'musicKey': subfield('r')}),
+    
+    '243': onwork.materialize('Collection', 
+                              'memberOf', 
+                              unique=values(subfield('a'), subfield('h'), subfield('k'), subfield('l'), subfield('m'), subfield('s')), 
+                              mr_properties={'title': subfield('a'), 'legalDate': subfield('d'), 'medium': subfield('h'), 'musicMedium': subfield('m'), 'musicKey': subfield('r')}),
 
-    '243$a': onwork.materialize('Collection', 
-                                'uniformMemberOf', 
-                                unique=all_subfields, 
-                                mr_properties={'name': subfield('a')}),
+    # Title(s) - replicate across both Work and Instance(s) 
+
+    '245$a': (onwork.rename(rel='title'), oninstance.rename(rel='title')),
+    '245$b': (onwork.rename(rel='titleRemainder'), oninstance.rename(rel='titleRemainder')),
+    '245$c': (onwork.rename(rel='titleStatement'), oninstance.rename(rel='titleStatement')),
+    '245$n': (onwork.rename(rel='titleNumber'), oninstance.rename(rel='titleNumber')),
+    '245$p': (onwork.rename(rel='titleName'), oninstance.rename(rel='titleName')),
 
-    '245$a': onwork.rename(rel='title'),
-    '245$b': onwork.rename(rel='subTitle'),
-    '245$c': onwork.rename(rel='titleStatement'),
     '245$f': onwork.rename(rel='inclusiveDates'),
     '245$h': onwork.rename(rel='medium'),
     '245$k': onwork.rename(rel='formDesignation'),
@@ -166,11 +171,36 @@ TRANSFORMS = {
     '340$c': oninstance.rename(rel='materialsApplied'),
     '340$d': oninstance.rename(rel='recordingTechnique'),
     '340$e': oninstance.rename(rel='physicalSupport'),
-    '351$a': oninstance.rename(rel='orgazationMethod'),
+    '351$a': oninstance.rename(rel='organizationMethod'),
     '351$b': oninstance.rename(rel='arrangement'),
     '351$c': oninstance.rename(rel='hierarchy'),
     '351$3': oninstance.rename(rel='materialsSpec'),
 
+# let's make some music! 
+#
+#    '382$a': onwork.materialize('Medium', 
+#                                'performanceMedium', 
+#                                unique=values(subfield('a')), 
+#                                mr_properties={'name': subfield('a')}),
+#
+#    '382$b': onwork.materialize('Medium', 
+#                                'featuredMedium', 
+#                                unique=values(subfield('b')), 
+#                                mr_properties={'name': subfield('b')}),
+#
+#    '382$p': onwork.materialize('Medium', 
+#                                'alternativeMedium', 
+#                                unique=values(subfield('f')), 
+#                                mr_properties={'name': subfield('f')}),
+#
+
+    '382$a': onwork.rename(rel='performanceMedium'),
+    '382$b': onwork.rename(rel='featuredMedium'),
+    '382$p': onwork.rename(rel='alternativeMedium'),
+
+    '382$s': onwork.rename(rel='numberOfPerformers'),
+    '382$v': onwork.rename(rel='mediumNote'),
+
     '490$a': onwork.rename(rel='seriesStatement'),
     '490$v': onwork.rename(rel='seriesVolume'),
 
@@ -245,17 +275,17 @@ TRANSFORMS = {
     '600': onwork.materialize('Person', 
                               'subject', 
                               unique=all_subfields,
-                              mr_properties={'name': subfield('a'), 'date': subfield('d'), 'hasAuthorityLink': subfield('0')}),
+                              mr_properties={'name': subfield('a'), 'numeration': subfield('b'), 'locationOfEvent': subfield('c'), 'date': subfield('d'), 'formSubdivision': subfield('v'), 'generalSubdivision': subfield('x'), 'chronologicalSubdivision': subfield('y'), 'geographicSubdivision': subfield('z'), 'hasAuthorityLink': subfield('0')}),
 
     '610': onwork.materialize('Organization', 
                               'subject', 
                               unique=all_subfields, 
-                              mr_properties={'name': subfield('a'), 'date': subfield('d'), 'hasAuthorityLink': subfield('0')}),
+                              mr_properties={'name': subfield('a'), 'subordinateUnit': subfield('b'), 'locationOfEvent': subfield('c'), 'date': subfield('d'), 'formSubdivision': subfield('v'), 'generalSubdivision': subfield('x'), 'chronologicalSubdivision': subfield('y'), 'geographicSubdivision': subfield('z'), 'hasAuthorityLink': subfield('0')}),
 
     '611': onwork.materialize('Meeting', 
                               'subject', 
                               unique=all_subfields, 
-                              mr_properties={'name': subfield('a'), 'date': subfield('d'), 'hasAuthorityLink': subfield('0')}),
+                              mr_properties={'name': subfield('a'), 'locationOfEvent': subfield('c'), 'date': subfield('d'), 'formSubdivision': subfield('v'), 'generalSubdivision': subfield('x'), 'chronologicalSubdivision': subfield('y'), 'geographicSubdivision': subfield('z'), 'hasAuthorityLink': subfield('0')}),
 
     #'610$d': onwork.rename(rel='date'),  #Note: there has been discussion about removing this, but we are not sure we get reliable ID.LOC lookups without it.  If it is removed, update augment.py
 
@@ -287,23 +317,64 @@ TRANSFORMS = {
                               unique=values(subfield('a'), subfield('b'), subfield('c'), subfield('g'), subfield('j'), subfield('q'), subfield('u')), 
                               mr_properties={'name': subfield('a'), 'numeration': subfield('b'), 'titles': subfield('c'), 'date': subfield('d'), 'hasAuthorityLink': subfield('0')}),
 
+    # 700$t - when MARC 700 are relationships not contributors
+    
+    '700$t': onwork.materialize('Work', 
+                                values('relatedTo', normalizeparse(subfield('i'))),
+                                unique=values(subfield('t')), 
+                                mr_properties={ifexists(subfield('a'), 'creator'): materialize('Person', unique=subfield('a'), mr_properties={'name': subfield('a'), 'date': subfield('d')}),
+                                               'title': subfield('t'),
+                                               'language': subfield('l')}),
+
     '710': onwork.materialize('Organization', 
                               values('contributor', normalizeparse(subfield('e')), normalizeparse(subfield('4'))), 
                               unique=values(subfield('a'), subfield('b'), subfield('c'), subfield('g'), subfield('j'), subfield('q'), subfield('u')), 
-                              mr_properties={'name': subfield('a'), 'date': subfield('d'), 'hasAuthorityLink': subfield('0')}),
+                              mr_properties={'name': subfield('a'), 'subordinateUnit': subfield('b'), 'date': subfield('d'), 'hasAuthorityLink': subfield('0')}),
+
+    # 710$t - when MARC 700 are relationships not contributors
+
+    '710$t': onwork.materialize('Work', 
+                                values('relatedTo', normalizeparse(subfield('i'))),
+                                unique=values(subfield('t')), 
+                                mr_properties={ifexists(subfield('a'), 'creator'): materialize('Organization', unique=subfield('a'), mr_properties={'name': subfield('a'), 'date': subfield('d')}),
+                                               'title': subfield('t'),
+                                               'language': subfield('l')}),
 
     '711': onwork.materialize('Meeting', 
                               values('contributor', normalizeparse(subfield('e')), normalizeparse(subfield('4'))), 
                               unique=values(subfield('a'), subfield('c'), subfield('d'), subfield('e'), subfield('q'), subfield('u')), 
                               mr_properties={'name': subfield('a'), 'date': subfield('d'), 'hasAuthorityLink': subfield('0')}),
 
-    '730': onwork.materialize('Collection', 'uniformMemberOf', unique=all_subfields, mr_properties={'name': subfield('a')}),
-    '830': onwork.materialize('Collection', 'uniformMemberOf', unique=all_subfields, mr_properties={'title': subfield('a')}),
+    # 711$t - when MARC 711 are relationships not contributors
 
-    #HeldItem is a refinement of Annotation
-    '852': oninstance.materialize('HeldItem', 'institution', unique=all_subfields, mr_properties={'holderType': 'Library', 'location': subfield('a'), 'subLocation': subfield('b'), 'callNumber': subfield('h'), 'code': subfield('n'), 'link': subfield('u'), 'streetAddress': subfield('e')}),
+    '711$t': onwork.materialize('Work', 
+                                values('relatedTo', normalizeparse(subfield('i'))),
+                                unique=values(subfield('t')), 
+                                mr_properties={ifexists(subfield('a'), 'creator'): materialize('Meeting', unique=subfield('a'), mr_properties={'name': subfield('a'), 'date': subfield('d')}),
+                                               'title': subfield('t'),
+                                               'language': subfield('l')}),
+
+
+    '830': onwork.materialize('Series', 
+                              'memberOf', 
+                              unique=values(subfield('a')), 
+                              mr_properties={'title': subfield('a'), 'subtitle': subfield('k'), 'volume': subfield('v'), 'number': subfield('n'), 'part': subfield('p'), }),
 
     '880$a': onwork.rename(rel='title'),
-    '856$u': oninstance.rename(rel='link'),
+    '856$u': oninstance.rename(rel='link', res=True),
+    }
+
+register_transforms("https://bibfra.me/tool/pybibframe/transforms#bflite", BFLITE_TRANSFORMS)
+
+MARC_TRANSFORMS = {
+    #HeldItem is a refinement of Annotation
+    '852': oninstance.materialize('HeldItem', 'institution', unique=all_subfields, mr_properties={'holderType': 'Library', 'location': subfield('a'), 'subLocation': subfield('b'), 'callNumber': subfield('h'), 'code': subfield('n'), 'link': subfield('u'), 'streetAddress': subfield('e')}),
 }
 
+register_transforms("https://bibfra.me/tool/pybibframe/transforms#marc", MARC_TRANSFORMS)
+
+#XXX This might instead be a job for collections.ChainMap
+TRANSFORMS = {}
+TRANSFORMS.update(BFLITE_TRANSFORMS)
+TRANSFORMS.update(MARC_TRANSFORMS)
+
